/*----------------------------------------------------------------------------------------

	Introduction to TigerSHARC Multiprocessing Systems Using VisualDSP++
	Maikel K.B., DSP-Europe 
	June 2002

	ID1.ASM

	This program demonstrates some of the Multiprocessing features handled by VisualDSP++ Tools.
	It shows the use of TigerSHARC in an MP System by performing the following tasks: 

		*	Setting up of control registers: SYSCON and SDRCON
	
		*	ID Checking routine to ensure that the code has been loaded into the right DSP
		
		*	It performs a vector interrupt (VIRPT) in DSP ID0 by writing the address of the 
			Interrupt Service Routine (ISR) to be executed in ID0 to the VIRPT register also
			in ID0. 

		*	It copies the contents of tx_ID1 to	the shared external	memory (shared_data 
			buffer declared in SDRAM) using DMA transfer. 
			
		*	Also, it performs a core write from tx_ID1 to rx_ID0 (internal memory of DSP ID1 
			via the MMS) using the Integer ALU (IALU) and circular buffers.
 	
		* 	After completion, it goes into an endless loop.

------------------------------------------------------------------------------------------*/


#define TAPS 16
#define MMS_ID0 0x2000000	// ID0 Multiprocessor Memory Space (MMS) offset value	
#define MMS_ID1 0x2400000	// ID1 Multiprocessor Memory Space (MMS) offset value		
#define VIRPT_REG 0x180730	// Vector Interrupt (VIRPT) register direct memory address

.extern shared_data;		// Data array defined in external shared memory (SDRAM) 
.extern rx_ID0;				// ID0 receive buffer (ID0.asm) for core (IALU) data transfer from ID1
.extern VIRPT_ISR_ID0;		// Address of Int. Serv. Rout. to be served by ID0 in response to 
							// VIRPT interrupt generated by ID1

.global rx_ID1;				// ID1 receive buffer for external port (DMA) data transfer from ID0


/* DM data section in Memory Block 1 */
.section data1;
.align 4;
.var Broadcast_rx_ID1[TAPS];	// ID1 broadcast receive buffer for broadcast transfer from ID0
.align 4;
.var rx_ID1[TAPS];				// ID1 receive buffer for external port (DMA) data transfer from ID0

/* DM data section in Memory Block 2 */
.section data2;
.align 4;
.var tx_ID1[TAPS]= "tx_ID1.dat";// ID1 transmit buffer for external port (DMA) data transfer to 
								// external shared memory (second half of buffer)

/* PM code in Memory Block 0 */
.section program;

start_ID1:
      
	/* Setting Up of Control Registers */

	j14 = j31 + 0x3a79e7;;			// Initial value after reset (0x279e7) modified 
    SYSCON = j14;;					// for 64-bit Bus width configuration

	j11 = j31 + 0x00005913;;		// Enable SDRAM: ENA=1, CAS=2 CL, pipedepth=0,			
	SDRCON = j11;;  				// page=512w, ref.rate=1200, trp=2, tras=5, init=1


	/* ID Checking Routine */

	xR0 = 0x3;;						// Routine to Check that code is being loaded in the right DSP (ID1)
	xR1 = SYSTAT;;					// read SYSTAT value 
	xr2 = 1;;						// DSP = ID1
	xR1 = FEXT R1 by R0;;			// Extract processor ID value (bits 3-0) from SYSTAT
	xr1 = r2 - r1;;					// is this DSP with ID1?
	
	if NAEQ, jump incorrect_ID;;	// if not correct, stop execution and enter endless loop


	/* DMA set up */

	XR0 = 0x00000000;;		
	XR1 = 0x00000000;;		
	XR2 = 0x00000000;;		
	XR3 = 0x00000000;;		

	XR1 = 0x00040004;;				// Count =4, modify = 4 for quad word accesses
	XR3 = 0x00000000;;				// Disable DMA
	DCD0 = XR3:0;;					// Set up DMA Channel 0 
	DCD1 = XR3:0;;					// Set up DMA Channel 1

	j0 = j31 + _dma_int;;			// Set up DMA0 & DMA1 interrupt vectors
	IVDMA0 = j0;;
	IVDMA1 = j0;;	
		
	j1 = MMS_ID0 + VIRPT_REG;;		// Write the address of the Interrupt Service Routine (ISR) to be
	xr0 = VIRPT_ISR_ID0 - MMS_ID0;;	// executed in ID0 (MMS_IDO) to the VIRPT register also in ID0
	[j1 += j31]= xr0;;				// MMS_IDO MUST be subtracted from ISR for the linker to resolve correctly

	/* Enable interrupts - DMA0, DMA1 and GLOBAL int. */

	xR0 = IMASKH;;					// Read value of IMASKH
	xR1 = 0x10000000;;				// Global interrupts
	xR2 = R1 or R0;;				// OR with contents of IMASKH
	IMASKH = xR2;;					// Write back to IMASKH

 	xR3 = IMASKL;;					// Read value of IMASKL
	xR4 = 0x0000C000;;				// DMA0 and DMA1 interrupts
	xR5 = R3 or R4;;				// OR with contents of IMASKL
	IMASKL = xR5;;					// Write back to IMASKL

	/* Set up and perform DMA transfers */

										// External port transmit - DMA0
	xr0 = tx_ID1;;						// xr0 = source index = int. mem. DSP_1
	xr1 = 0x00100004;;					// count = 0x10, modify = 4 (quadword)
	xr3 = 0x47000000;;					// int mem,prio=norm,2D=no,word=quad,int=yes,RQ=enbl,chain=no
	DCS0 = xr3:0;;						// Set up source for transfer

	yr0 = shared_data + TAPS;;			// xr0 = destination index = ext. mem.(SDRAM)
	yr1 = 0x00100004;;					// count = 0x10, modify = 4 (quadword)
	yr3 = 0x87000000;;					// ext mem,prio=norm,2D=no,word=quad,int=yes,RQ=enbl,chain=no
	DCD0 = yr3:0;;						// Set up destination and start external port DMA0 transfer

	
	/* Core Transfer: ID1 to ID0 data transfer using the Integer ALU (IALU) */

	jB0 = tx_ID1;;			// Define circular buffer base register for trasnmit buffer in ID1 
	j0 = jB0;;				// Set circular buffer index value equals to base address 			
	jL0 = TAPS;;			// Set circular buffer length

	jB1 = rx_ID0;;			// Define circular buffer base register for receive buffer in ID0 
	j1 = jB1;;				// Set circular buffer index value equals to base address 			
	jL1 = TAPS;;			// Set circular buffer length 

	j4  = 1;;				// Circular buffer modify value 
	LC0 = TAPS;;			// Initialize loop counter

	write_ext:	xr0 = CB[j0 += j4];;	// read data from tx_ID1 
				CB[j1 += j4] = xr0;;	// write data to rx_ID0

	if NLC0E, jump write_ext;;	// loop back until all data has been transfered

/* Terminate and wait */
wait1:   jump wait1;;


_dma_int:	

	nop;;nop;;nop;;rti(ABS);;				//	DMA0 & DMA1 Interrupt vector

incorrect_ID:	

	nop;;nop;;nop;;jump incorrect_ID;;  	// ERROR - Code loaded into wrong DSP. Check DSP ID Number!!! 
